% This page corresponds to Tutorial 6 (Advanced IC Analysis)
% The scripts basically follow the exercises presented in the 
% corresponding Tutorial presentation. Some scripts have been 
% enhanced by looping through conditions or ICs to help find 
% interesting activity and activity differences in the dataset.
% Try writing the assigned scripts yourself, only using the scripts below
% to help if you get stuck. 
% If you do need to use the scripts below, proceed line-by-line in your 
% chosen section and be sure you understand what each line does before
% pasting it into the Matlab window. 
% BEGIN --------------------------------------------------------------

eeglab % start eeglab
% make sure memory options are set for more than one dataset in memory!
pop_editoptions( 'option_storedisk', 0, 'option_savetwofiles', 1, 'option_saveica', 0, 'option_single', 1, 'option_memmapdata', 0, 'option_computeica', 1, 'option_scaleicarms', 1, 'option_rememberfolder', 1);

basedir = 'C:\\Users\\julie\\Documents\\Workshops\\Finland\\data\\';
datset = 'stern.set'
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SEARCH EEG.event structure for specific events
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Objective:  
% 1)Find all Memorize letters that were preceded by an ignore letter
% 2)Find all Memorize letters that were preceded by a memorize letter
% hint: memorize event codes are a single letters

% load stern.set from the GUI
EEG = pop_loadset( 'filename', datset, 'filepath', basedir);
[ALLEEG, EEG, CURRENTSET] = eeg_store( ALLEEG, EEG, 0 );

epochidxM=[]; % Mem preceded by a mem letter
epochidxG=[]; % Mem preceded by an ignore letter
for ev = 2:length(EEG.event) % since we search ev-1, must start at 2
	if length(EEG.event(ev).type) == 1 & length(EEG.event(ev-1).type) == 1
		epochidxM = [epochidxM, ev]; % save this event
    elseif length(EEG.event(ev).type) == 1 & EEG.event(ev-1).type(1) == 'g'
		epochidxG = [epochidxG, ev]; % save this event
	end;
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EPOCH data using collected event indices
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (make epochs long enough to verify the event type preceding each time-locking
% event)
[EEG, indices] = pop_epoch( EEG, [], [-2 2],'eventindices',epochidxG);
[ALLEEG EEG CURRENTSET] = pop_newset(ALLEEG, EEG, 1, 'setname','Mem after Ignore letter','gui', 'off'); 
%EEG = pop_autorej(EEG,  'nogui', 'on'); % remove noisy epochs
[ALLEEG EEG CURRENTSET] = pop_newset(ALLEEG, EEG,CURRENTSET, 'retrieve',1);
[EEG, indices] = pop_epoch( EEG, [], [-2 2],'eventindices',epochidxM);
[ALLEEG EEG CURRENTSET] = pop_newset(ALLEEG, EEG, 1, 'overwrite','on', 'setname','Mem after Mem letter','gui', 'off'); 
%EEG = pop_autorej(EEG,  'nogui', 'on'); % remove noisy epochs
eeglab redraw
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PLOT ERSP images for selected ICs to detect any sig differences
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% list suspected brain ICs:
gdcomps = [2:11];%,13,17,18,19,20,23,24,25,26,28,29,31,35,39,45];
for ic = 1:length(gdcomps) % RT sort
    [ersp(:,:,ic),itc(:,:,ic),powbase(:,ic),times,freqs,erspboot(:,:,ic),itcboot(:,:,ic)] =  ...
       newtimef({ALLEEG(1).icaact(gdcomps(ic),:),ALLEEG(2).icaact(gdcomps(ic),:)},...
       EEG.pnts, [EEG.xmin EEG.xmax]*1000, EEG.srate, [3 .5],'type', 'phasecoher',...
       'topovec', EEG.icawinv(:,gdcomps(ic)),'elocs', EEG.chanlocs, 'chaninfo',...
       EEG.chaninfo,'title',{['IC ',int2str(gdcomps(ic)),' M a M'],...
       ['IC ',int2str(gdcomps(ic)),' M a Ig']},'alpha',.01,'padratio', 1, ...
       'plotphase','off','winsize',EEG.srate);
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  SORT ERP images on RT latency, in/out, memory load
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load stern.set, epoch on Probe letter:
EEG = pop_loadset( 'filename', datset, 'filepath', basedir);
[ALLEEG, EEG, CURRENTSET] = eeg_store( ALLEEG, EEG, 0 );
EEG = pop_epoch( EEG, {'rB'  'rC'  'rD'  'rF'  'rG'  'rH'  'rJ'  'rK'  'rL'  'rM'  'rN'  'rP'  'rQ'  'rR'  'rS'  'rT'  'rV'  'rW'  'rX'  'rY'  'rZ'  }, [-4  4], 'newname', 'Sternberg Probe', 'epochinfo', 'yes');
[ALLEEG EEG CURRENTSET] = pop_newset(ALLEEG, EEG, 1, 'overwrite', 'on', 'gui', 'off'); 
EEG = pop_autorej(EEG,  'nogui', 'on'); % remove noisy epochs
eeglab redraw
gdcomps= [6,8,10,13,17,19,23,26,28,45]; % list brain ICs 
% Response-sorted ERP images (activation and alpha power plots):
for ic = 1:length(gdcomps)
    figure;
    sbplot(1,2,1);
    erpimage( EEG.icaact([gdcomps(ic)], :), eeg_getepochevent( EEG,...
       { 'in', 'out'},[], 'latency'), linspace(EEG.xmin*1000, EEG.xmax*1000, EEG.pnts),...
       ['Comp. ',int2str(gdcomps(ic))], 5, 1 , 'yerplabel','', 'erp', 'on',...
       'limits',[-2000 2000 NaN NaN NaN NaN NaN NaN] , 'cbar', 'on',...
       'topo', { mean(EEG.icawinv(:,[gdcomps(ic)]),2) EEG.chanlocs EEG.chaninfo } );
    sbplot(1,2,2);
    erpimage( EEG.icaact([gdcomps(ic)], :), eeg_getepochevent( EEG,...
       { 'in', 'out'},[], 'latency'), linspace(EEG.xmin*1000, EEG.xmax*1000, EEG.pnts),...
       ['Comp. ',int2str(gdcomps(ic))], 5, 1 , 'yerplabel','', 'erp', 'on',...
       'limits',[-2000 2000 NaN NaN NaN NaN NaN NaN] , 'cbar', 'on','plotamps', 'on',...
       'coher',[8 12 0.01] ,'topo', { mean(EEG.icawinv(:,[gdcomps(ic)]),2) EEG.chanlocs EEG.chaninfo } );
end;
% plot type-sorted ERP images with rt overplotted
% sort on response TYPE ('in' or 'out' of set)
for ic = 1:length(gdcomps) 
    figure;
    sbplot(1,2,1);
     [outdata,outvar,outtrials,limits,axhndls,erp,amps,cohers,cohsig,ampsig,outamps,...
        phsangls,phsamp,sortidx,erpsig]=erpimage( EEG.icaact([gdcomps(ic)], :), eeg_getepochevent( EEG, { 'in', 'out'},[], 'type'), linspace(EEG.xmin*1000, EEG.xmax*1000, EEG.pnts), ['Comp. ',int2str(gdcomps(ic))], 5, 1 , 'yerplabel','', 'erp', 'on', 'limits',[-300 2000 NaN NaN NaN NaN NaN NaN] , 'cbar', 'on','topo', { mean(EEG.icawinv(:,[gdcomps(ic)]),2) EEG.chanlocs EEG.chaninfo } );
    sbplot(1,2,2);
    erpimage( EEG.icaact([gdcomps(ic)], :), eeg_getepochevent( EEG, { 'in', 'out'},[], 'type'),...
       linspace(EEG.xmin*1000, EEG.xmax*1000, EEG.pnts), ['Comp. ',int2str(gdcomps(ic))],...
       5, 1 , 'yerplabel','', 'erp', 'on', 'limits',[-300 1000 NaN NaN NaN NaN NaN NaN] ,...
       'cbar', 'on', 'plotamps', 'on', 'coher',[8 12 0.01] ,'topo',...
       { mean(EEG.icawinv(:,[gdcomps(ic)]),2) EEG.chanlocs EEG.chaninfo } );
end;

% sort on trial LOAD (3,5,or 7 memorize letters)
for ic = 1:length(gdcomps)
    figure;
    sbplot(1,2,1);
    [outdata,outvar,outtrials,limits,axhndls,erp,amps,cohers,cohsig,ampsig,outamps,...
       phsangls,phsamp,sortidx,erpsig]=erpimage( EEG.icaact([gdcomps(ic)], :),...
       eeg_getepochevent( EEG, { 'rB', 'rC', 'rD', 'rF', 'rG', 'rH', 'rJ', 'rK', 'rL', 'rM', 'rN', 'rP', 'rQ', 'rR', 'rS', 'rT', 'rV', 'rW', 'rX', 'rY', 'rZ'},[], 'load'),...
       linspace(EEG.xmin*1000, EEG.xmax*1000, EEG.pnts), ['Comp. ',int2str(gdcomps(ic))],...
       5, 1 , 'yerplabel','', 'erp', 'on', 'limits',[-3000 1000 NaN NaN NaN NaN NaN NaN] ,...
       'cbar', 'on','topo', { mean(EEG.icawinv(:,[gdcomps(ic)]),2) EEG.chanlocs EEG.chaninfo } );
    sbplot(1,2,2);
    [outdata,outvar,outtrials,limits,axhndls,erp,amps,cohers,cohsig,ampsig,outamps,...
       phsangls,phsamp,sortidx,erpsig]=erpimage( EEG.icaact([gdcomps(ic)], :),...
       eeg_getepochevent( EEG, { 'rB', 'rC', 'rD', 'rF', 'rG', 'rH', 'rJ', 'rK', 'rL', 'rM', 'rN', 'rP', 'rQ', 'rR', 'rS', 'rT', 'rV', 'rW', 'rX', 'rY', 'rZ'},[], 'load'),...
       linspace(EEG.xmin*1000, EEG.xmax*1000, EEG.pnts), ['Comp. ',int2str(gdcomps(ic))],...
       5, 1 , 'yerplabel','', 'erp', 'on', 'limits',[-3000 1000 NaN NaN NaN NaN NaN NaN] ,...
       'cbar', 'on', 'plotamps', 'on', 'coher',[8 12 0.01],'topo',...
       { mean(EEG.icawinv(:,[gdcomps(ic)]),2) EEG.chanlocs EEG.chaninfo } );
end;

% Same ERP images with RT overplotted---------------
% collect rt latency to overplot using 'auxvar' (below)
rtlat = []; 
for ep = 1:length(EEG.epoch)
    for ev = 1:length(EEG.epoch(ep).eventtype)
        if EEG.epoch(ep).eventtype{ev}(1) == 'r'
            %problat = [problat EEG.epoch(ep).eventlatency{ev}-EEG.epoch(ep).eventlatency{ev-1}];
            if ev == length(EEG.epoch(ep).eventtype)
                rtlat = [rtlat 0];
            end;
        elseif strcmp(EEG.epoch(ep).eventtype{ev},'in') |  strcmp(EEG.epoch(ep).eventtype{ev},'out')
            rtlat = [rtlat EEG.epoch(ep).eventlatency{ev}-EEG.epoch(ep).eventlatency{ev-1}];
        end;
    end;
end;

% sort on trial LOAD (3,5,or 7 memorize letters) with rt overplotted
for ic = 1:length(gdcomps)
    figure;
    sbplot(1,2,1);
    [outdata,outvar,outtrials,limits,axhndls,erp,amps,cohers,cohsig,ampsig,outamps,...
       phsangls,phsamp,sortidx,erpsig]=erpimage( EEG.icaact([gdcomps(ic)], :),...
       eeg_getepochevent( EEG, { 'rB', 'rC', 'rD', 'rF', 'rG', 'rH', 'rJ', 'rK', 'rL', 'rM', 'rN', 'rP', 'rQ', 'rR', 'rS', 'rT', 'rV', 'rW', 'rX', 'rY', 'rZ'},[], 'load'),...
       linspace(EEG.xmin*1000, EEG.xmax*1000, EEG.pnts), ['Comp. ',int2str(gdcomps(ic))],...
       5, 1 , 'yerplabel','', 'erp', 'on', 'limits',[-2000 2000 NaN NaN NaN NaN NaN NaN] ,...
       'cbar', 'on','topo', { mean(EEG.icawinv(:,[gdcomps(ic)]),2) EEG.chanlocs EEG.chaninfo },'auxvar',rtlat );
    sbplot(1,2,2);
    [outdata,outvar,outtrials,limits,axhndls,erp,amps,cohers,cohsig,ampsig,outamps,...
       phsangls,phsamp,sortidx,erpsig]=erpimage( EEG.icaact([gdcomps(ic)], :),...
       eeg_getepochevent( EEG, { 'rB', 'rC', 'rD', 'rF', 'rG', 'rH', 'rJ', 'rK', 'rL', 'rM', 'rN', 'rP', 'rQ', 'rR', 'rS', 'rT', 'rV', 'rW', 'rX', 'rY', 'rZ'},[], 'load'),...
       linspace(EEG.xmin*1000, EEG.xmax*1000, EEG.pnts), ['Comp. ',int2str(gdcomps(ic))],...
       5, 1 , 'yerplabel','', 'erp', 'on', 'limits',[-2000 2000 NaN NaN NaN NaN NaN NaN] ,...
       'cbar', 'on', 'plotamps', 'on', 'coher',[8 12 0.01],'topo',...
       { mean(EEG.icawinv(:,[gdcomps(ic)]),2) EEG.chanlocs EEG.chaninfo } ,'auxvar',rtlat);
end;
% plot type-sorted ERP images with rt overplotted
for ic = 1:length(gdcomps) 
    figure;
    sbplot(1,2,1);
     [outdata,outvar,outtrials,limits,axhndls,erp,amps,cohers,cohsig,ampsig,outamps,...
        phsangls,phsamp,sortidx,erpsig]=erpimage( EEG.icaact([gdcomps(ic)], :),...
        eeg_getepochevent( EEG, { 'in', 'out'},[], 'type'),...
        linspace(EEG.xmin*1000, EEG.xmax*1000, EEG.pnts),...
        ['Comp. ',int2str(gdcomps(ic))], 5, 1 , 'yerplabel','', 'erp', 'on',...
        'limits',[-300 2000 NaN NaN NaN NaN NaN NaN] , 'cbar', 'on',...
        'topo', { mean(EEG.icawinv(:,[gdcomps(ic)]),2) EEG.chanlocs EEG.chaninfo },...
        'auxvar',rtlat );
    sbplot(1,2,2);
    erpimage( EEG.icaact([gdcomps(ic)], :), eeg_getepochevent( EEG,...
       { 'in', 'out'},[], 'type'), linspace(EEG.xmin*1000, EEG.xmax*1000, EEG.pnts),...
       ['Comp. ',int2str(gdcomps(ic))], 5, 1 , 'yerplabel','', 'erp', 'on',...
       'limits',[-300 2000 NaN NaN NaN NaN NaN NaN] , 'cbar', 'on', 'plotamps', 'on',...
       'coher',[8 12 0.01] ,'topo',...
       { mean(EEG.icawinv(:,[gdcomps(ic)]),2) EEG.chanlocs EEG.chaninfo },'auxvar',rtlat );
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Sort on alpha phase, then sort another ERP image using output 'sortidx':
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
smooth = 20; tmlims = [-800 800];

% choose ICs to plot:
ic1= 5; ic2 = 3;
tmsort = [-75]; % window center for phase-sort

% plot with no sort first:
cax1 = [-1.5 1.5]; figure;erpimage( EEG.icaact([ic1], :), ones(1, EEG.trials)*EEG.xmax*1000, linspace(EEG.xmin*1000, EEG.xmax*1000, EEG.pnts),['Comp. ',int2str(ic1)], smooth, 1 ,  'yerplabel','', 'erp', 'on', 'limits',[tmlims NaN NaN NaN NaN NaN NaN] , 'cbar', 'on','topo', { mean(EEG.icawinv(:,[ic1]),2) EEG.chanlocs EEG.chaninfo }, 'caxis',cax1 );
cax2 = [-1.5 1.5]; figure;erpimage(EEG.icaact([ic2], :),ones(1, EEG.trials)*EEG.xmax*1000 , linspace(EEG.xmin*1000, EEG.xmax*1000, EEG.pnts), ['Comp. ',int2str(ic2)], smooth, 1  , 'yerplabel','', 'erp', 'on', 'limits',[tmlims NaN NaN NaN NaN NaN NaN] , 'cbar', 'on','topo', { mean(EEG.icawinv(:,[ic2]),2) EEG.chanlocs EEG.chaninfo }, 'caxis',cax2 );

% now valsort the first IC (collect output):
figure;[outdata,outvar,outtrials,limits,axhndls,erp,amp,cohers,cohsig,ampsig,outamps, phsangls,phsamp,sortidx,erpsig]=erpimage( EEG.icaact([ic1], :), ones(1, EEG.trials)*EEG.xmax*1000, linspace(EEG.xmin*1000, EEG.xmax*1000, EEG.pnts), ['Comp. ',int2str(ic1)], smooth, 1 , 'yerplabel','', 'erp', 'on', 'limits',[tmlims NaN NaN NaN NaN NaN NaN] , 'cbar', 'on', 'phasesort',[tmsort 0 8 12] ,'topo', { mean(EEG.icawinv(:,[ic1]),2) EEG.chanlocs EEG.chaninfo }, 'caxis',cax1 );

% resort data from IC2:
dat =  squeeze(EEG.icaact([ic2], :,sortidx)); % plot IC2 with new sort order:
figure;[outdata,outvar,outtrials,limits,axhndls,erp,amp,cohers,cohsig,ampsig,outamps, phsangls,phsamp,sortidx2,erpsig]=erpimage(dat,ones(1, EEG.trials)*EEG.xmax*1000 , linspace(EEG.xmin*1000, EEG.xmax*1000, EEG.pnts), ['Comp. ',int2str(ic2)], smooth, 1  , 'yerplabel','', 'erp', 'on', 'limits',[tmlims NaN NaN NaN NaN NaN NaN] , 'cbar', 'on','topo', { mean(EEG.icawinv(:,[ic2]),2) EEG.chanlocs EEG.chaninfo }, 'caxis',cax2 );

% plot same sort order with power values instead of activation:
figure;[outdata,outvar,outtrials,limits,axhndls,erp,amp,cohers,cohsig,ampsig,outamps, phsangls,phsamp,sortidx2,erpsig]=erpimage(dat,ones(1, EEG.trials)*EEG.xmax*1000 , linspace(EEG.xmin*1000, EEG.xmax*1000, EEG.pnts), ['Comp. ',int2str(ic2)], smooth, 1  , 'yerplabel','', 'erp', 'on', 'limits',[-400 1000 NaN NaN NaN NaN NaN NaN] , 'cbar', 'on','topo', { mean(EEG.icawinv(:,[ic2]),2) EEG.chanlocs EEG.chaninfo }, 'coher',[8 12 0.01],'plotamps','on');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 