% this page corresponds to Tutorial 8 (Build an EEGLAB STUDY, Julie Onton)
% Try writing the assigned scripts yourself, only using the scripts below
% to help if you get stuck. 
% If you do need to use the scripts below, proceed line-by-line in your 
% chosen section and be sure you understand what each line does before
% pasting it into the Matlab window. 
% * IMPORTANT: directory paths will be specific for your personal computer, 
%              please edit appropriately in all scripts provided in the 
%              variable called 'basedir' ***

% BEGIN STUDY scripting:--------------------------------------------------------------

%%%  Variables:  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set correct path to STUDY:
basedir = '/Volumes/donnees/data/STUDYstern'; % make specific to your data path
setnames = {'Memorize.set','Ignore.set','Probe.set'};
conds={'memorize','ignore','probe'};
subjs = {'S01','S02','S03','S04','S05','S06','S07','S08','S09','S10','S11','S12','S13'};
studyname = 'Sternberg';
taskname = 'Sternberg';
savename = 'sterntest.study';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% BUILD a STUDY:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Open eeglab:
[ALLEEG EEG CURRENTSET ALLCOM] = eeglab; % open eeglab
% make sure memory options are for only one dataset up at once:
pop_editoptions( 'option_storedisk', 1, 'option_savetwofiles', 1, 'option_saveica', 1, 'option_single', 0, 'option_memmapdata', 0, 'option_computeica', 1, 'option_scaleicarms', 0, 'option_rememberfolder', 1);% saves a file 'eeg_options.m' to your current working directory

STUDY=[];ALLEEG=[]; EEG=[];CURRENTSET=[];CURRENTSTUDY=[];
index = 1;
for subj = 1:length(subjs)
    for cond = 1:length(setnames)
        datset = fullfile( basedir, subjs{subj}, setnames{cond});% concatenate strings               
		[STUDY ALLEEG] = std_editset( STUDY, ALLEEG,'name',studyname,'task', taskname,...
            'commands',{{'index',index,'load',datset}, {'index',index,'subject',subjs{subj}},...
            {'index',index, 'session', 1},...
           {'index',index, 'group', 1},{'index',index,'condition',conds{cond}},...
           {'index',index,'dipselect',.15}}, 'updatedat','off',...
            'savedat', 'off', 'filename', [basedir,savename]); %,'inbrain','on');
        index = index+1; % update set file index
    end;
end;
CURRENTSTUDY = 1; EEG = ALLEEG; CURRENTSET = [1:length(EEG)]; % reassign EEGLAB variables
STUDY.design = []; % remove design to rebuild it below
[STUDY, ALLEEG] = std_checkset(STUDY, ALLEEG); % check STUDY for consistency
eeglab redraw

% Save STUDY :---------------
[STUDY EEG] = pop_savestudy( STUDY, EEG,  'savemode', 'standard','filename',savename,'filepath',basedir);
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (Optional) BUILD a STUDY (input pre-selected components for all subjects):
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% pop_editoptions('option_storedisk', 1, 'option_savetwofiles', 1, 'option_saveica', 0, 'option_single', 0, 'option_computeica', 1, 'option_scaleicarms', 0, 'option_rememberfolder', 1);% saves a file 'eeg_options.m' to your current working directory
% 
% gdcomps{1} = []; % list ICs of interest for subject 1
% gdcomps{2} = []; % list ICs of interest for subject 2
% gdcomps{3} = []; % list ICs of interest for subject 3
% gdcomps{4} = []; % list ICs of interest for subject 4
% gdcomps{5} = []; % list ICs of interest for subject 5
% gdcomps{6} = []; % list ICs of interest for subject 6
% gdcomps{7} = []; % list ICs of interest for subject 7
% gdcomps{8} = []; % list ICs of interest for subject 8
% gdcomps{9} = []; % list ICs of interest for subject 9
% % etc for all subjects
% 
% STUDY=[];ALLEEG=[]; EEG=[];
% index = 1;
% for subj = 1:length(subjs)
%     for cond = 1:length(setnames)
%         datset = [basedir,subjs{subj},'\',setnames{cond}];% concatenate strings
% 
% 		[STUDY ALLEEG] = std_editset( STUDY, ALLEEG,'name',studyname,'task', taskname,...
%             'commands',{{'index',index,'load',datset}, {'index',index,'subject',subjs{subj}},...
%             {'index',index, 'session', 1},...
%            {'index',index, 'group', 1},{'index',index,'condition',conds{cond}},...
%            {'comps',gdcomps{subj}}}, 'updatedat','off',...
%             'savedat', 'off', 'filename', [basedir,savename]);
% 
%          index = index+1; % update set file index
%         CURRENTSTUDY = 1; EEG = ALLEEG; CURRENTSET = [1:length(EEG)]; % reassign EEGLAB variables
%         [STUDY, ALLEEG] = std_checkset(STUDY, ALLEEG); % check STUDY for consistency
%     end;
% end;
% eeglab redraw

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PRECOMPUTE data measures for COMPONENTS:
% Choose all measures to compute, (compute all, and select precluster measures later)
% Recommend: specify alpha for ERSPs
% NOTE: all IC measures are precomputed for you, so you do not need to 
% run this line at the workshop!

[STUDY ALLEEG] = std_precomp(STUDY, ALLEEG,  'components', 'erp', 'on', 'rmbase',[-200 0] , 'scalp', 'on', 'spec', 'on', 'specparams',{}, 'ersp', 'on', 'erspparams',{ 'cycles',[3 0.5] , 'nfreqs',100, 'freqs',[3 70] , 'alpha',0.01}, 'itc', 'on');
[STUDY, ALLEEG] = std_checkset(STUDY, ALLEEG);
CURRENTSTUDY = 1; EEG = ALLEEG; CURRENTSET = [1:length(EEG)];

% RE-Save STUDY:---------------
[STUDY EEG] = pop_savestudy( STUDY, EEG,  'savemode', 'resave');% path and filename in STUDY structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PRECLUSTER the data into a single matrix:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% use spectra, erp, dipoles, ersp, itc----------

parentclust = 1; % cluster 1 is always full parent cluster

[STUDY ALLEEG] = std_preclust(STUDY, ALLEEG, 1,{ 'spec', 'npca',5, 'norm',1,...
    'weight',1, 'freqrange',[3 25] },{ 'erp', 'npca',6, 'norm',1, 'weight',1,...
    'timewindow',[0 400] },{ 'scalp', 'npca',10, 'norm',1, 'weight',1, 'abso',1},...
    { 'dipoles', 'norm',1, 'weight',10},{ 'ersp', 'npca',20, 'freqrange',[3 30] ,...
    'timewindow',[0 600] , 'norm',1, 'weight',1},{ 'itc', 'npca',6,'freqrange',[3 30],...
    'timewindow',[0 400] , 'norm',1, 'weight',1});

CURRENTSTUDY = 1; EEG = ALLEEG; CURRENTSET = [1:length(EEG)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CLUSTER the data:------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nclusts = 24; % choose how many clusters to create (start high and work down if necessary)
[STUDY] = pop_clust(STUDY, ALLEEG, 'algorithm','kmeans','clus_num',  nclusts );

% OR specify a standard dev. cut off. Tip: examine the results with and without this parameter
[STUDY] = pop_clust(STUDY, ALLEEG, 'algorithm','kmeans','clus_num',  nclusts , 'outliers',  3 );

% save the STUDY with new clusters:-----------------------
[STUDY EEG] = pop_savestudy( STUDY, EEG,  'savemode', 'resave');% path and filename in STUDY structure

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% TIP : with very large datasets, a quicker way to load the STUDY is to save a .mat of 
% your STUDY variables and load that instead of loading STUDY from the gui:
% to save your STUDY, type the following lines:-------------
save_command = ['save ',basedir,'STUDYinfo.mat'];
eval(save_command)

% then, when you start matlab again later, type:-------------
load_command = ['load ',basedir,'STUDYinfo.mat'];
eval(load_command)
eeglab redraw;  % update gui with new dataset information

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ADVANCED exercise: 
% How do you know how many PCA dimensions to use?
% Call in raw data and run PCA, then plot singular values:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cond = 1; % experimental condition (corresponds to STUDY.condition)

clear logersp
for ic = 1:size(STUDY.cluster(1).sets,2)
    [logersp(:,:,ic), logfreqs, timevals, params, baseersp] = std_readersp(ALLEEG, STUDY.cluster(1).sets(cond,ic), STUDY.cluster(1).comps(ic), [0 1000],[0 40] );
end;
ersp2d = reshape(logersp,size(logersp,1)*size(logersp,2),size(logersp,3));

npcs = 50; % limit the number and speed up PCA
[pc,eigvec,sv] = runpca(double(ersp2d),npcs); % decompose ERSP x ICs

% PLOT the singluar values:
figure; sbplot(2,2,1); plot(max(sv));set(gca,'xlim',[1 size(sv,1)]);
title('Raw singular values'); xlabel('PCs'); 

sbplot(2,2,2);plot(max(sv)/max(sv(:)));hold on;
set(gca,'xlim',[1 size(sv,1)]);
title('Normalized singular values'); xlabel('PCs'); 
plot([get(gca,'xlim')],[.1 .1],'r-');% line showing 10% of max

sbplot(2,2,3); imagesc(eigvec); xlabel('PCs');  title('ERSP templates'); 
ylabel('ERSP time/freq points');cbar; 

sbplot(2,2,4); imagesc(pc); title('Component weightings');  
ylabel('PCs'); xlabel('Trials'); cbar; 
textsc('ERSP PCA dimensions','title');

% same thing for spectra:---------------------------------------
clear spec
for ic = 1:size(STUDY.cluster(1).sets,2)
    [spec(ic,:), freqs] = std_readspec(ALLEEG, STUDY.cluster(1).sets(cond,ic),  STUDY.cluster(1).comps(ic), [0 40], 0);  
end;

npcs = 50; % limit the number and speed up PCA
[pc,eigvec,sv] = runpca(double(spec'),npcs); % decompose ERSP x ICs

% PLOT the singluar values:
figure; sbplot(2,2,1); plot(max(sv)); set(gca,'xlim',[1 size(sv,1)]);
title('Raw singular values'); xlabel('PCs'); 

sbplot(2,2,2);plot(max(sv)/max(sv(:)));hold on;set(gca,'xlim',[1 size(sv,1)]);
title('Normalized singular values'); xlabel('PCs'); 
plot([get(gca,'xlim')],[.1 .1],'r-');% line showing 10% of max

sbplot(2,2,3); imagesc(eigvec); xlabel('PCs');  title('Spectral templates'); 
ylabel('Freq points');cbar; 

sbplot(2,2,4); imagesc(pc); title('Component weightings');  ylabel('PCs'); 
xlabel('Trials'); cbar; 
textsc('Spectral PCA dimensions','title');

% same thing for ITC:---------------------------------------
clear logitc
for ic = 1:size(STUDY.cluster(1).sets,2)
    [logitc(:,:,ic), logfreqs, times] = std_readitc(ALLEEG, STUDY.cluster(1).sets(cond,ic),  STUDY.cluster(1).comps(ic), [0 400],[0 25] );
end;
itc2d = reshape(logitc,size(logitc,1)*size(logitc,2),size(logitc,3));
itc2d = real(itc2d); % use magnitude only

npcs = 50; % limit the number and speed up PCA
[pc,eigvec,sv] = runpca(double(itc2d),npcs); % decompose ERSP x ICs

% PLOT the singluar values:
figure; sbplot(2,2,1); plot(max(sv));set(gca,'xlim',[1 size(sv,1)]);
title('Raw singular values'); xlabel('PCs'); 

sbplot(2,2,2);plot(max(sv)/max(sv(:)));hold on;set(gca,'xlim',[1 size(sv,1)]);
title('Normalized singular values'); xlabel('PCs'); 
plot([get(gca,'xlim')],[.1 .1],'r-'); % line showing 10% of max

sbplot(2,2,3); imagesc(eigvec);xlabel('PCs');  title('ITC templates');  
ylabel('ERSP time/freq points');cbar; 

sbplot(2,2,4); imagesc(pc); title('Component weightings');  ylabel('PCs');  
xlabel('Trials'); cbar; 
textsc('ITC PCA dimensions','title');
