% this page corresponds to Tutorial 8:
% "STUDY analysis I", by Julie Onton
% Below are novice, intermediate and advanced exercises. 
% Try writing the assigned scripts yourself, only using the scripts below
% to help if you get stuck. 
% If you do need to use the scripts below, proceed line-by-line in your 
% chosen section and be sure you understand what each line does before
% pasting it into the Matlab window. 
% * IMPORTANT: directory paths will be specific for your personal computer, 
%              please edit appropriately in all scripts provided in the 
%              variable called 'datpath' ***

% BEGIN STUDY scripting:--------------------------------------------------------------

%%%  Variables:  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

basedir = 'C:\Users\julie\Documents\Workshops\Taiwan\STUDY\'; % make specific to your data path
setnames = {'Memorize.set','Ignore.set','Probe.set'};
subjs = {'S01','S02','S03','S04','S05','S06','S07','S08','S09','S10','S11','S12','S13'};
studyname = 'Sternberg';
taskname = 'Sternberg';
savename = 'stern.study';

%%% Open eeglab:
[ALLEEG EEG CURRENTSET ALLCOM] = eeglab; % open eeglab
% make sure memory options are for only one dataset up at once:
pop_editoptions( 'option_storedisk', 1, 'option_savetwofiles', 1, 'option_saveica', 1, 'option_single', 0, 'option_memmapdata', 0, 'option_computeica', 1, 'option_scaleicarms', 0, 'option_rememberfolder', 1);% saves a file 'eeg_options.m' to your current working directory

%load your STUDY:
[STUDY ALLEEG] = pop_loadstudy('filename', savename, 'filepath', basedir);
CURRENTSTUDY = 1; EEG = ALLEEG; CURRENTSET = [1:length(EEG)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PLOT ERSPs for a cluster:------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clust = 4; % choose a cluster 
cond = 1; % choose experimental condition

clear logersp
STUDY = pop_erspparams(STUDY, 'subbaseline','off','topotime',NaN,'topofreq',NaN);
[STUDY logersp timevals logfreqs pgroup pcond pinter] = std_erspplot(STUDY,ALLEEG,...
   'clusters',clust, 'plotsubjects', 'on' );

figure; clim = 3; % standardize color limits 
row = ceil(sqrt(length(STUDY.cluster(clust).comps))); % define number of rows
col = ceil(sqrt(length(STUDY.cluster(clust).comps))); % define number of columns

for ic = 1:size(logersp{cond},3)
    sbplot(row,col,ic); 
    imagesclogy(timevals, logfreqs, logersp{cond}(:,:,ic));cbar;
    %set(gca,'clim', [-clim clim]); % adjust the color limits 
    set(gca,'ydir','norm'); % plot low freqs at the bottom
    title(['IC ',int2str(STUDY.cluster(clust).comps(ic))]);
end;  
textsc(['Cluster ',int2str(clust)],'title');
cbar; % include a colorbar 

% plot the mean for comparison:
sbplot(row,col,ic+1)
imagesclogy(timevals, logfreqs, mean(logersp{cond},3));cbar;
set(gca,'ydir','norm'); % plot low freqs at the bottom
title(['Mean: Cluster ',int2str(clust)]);

textsc(['Cluster ',int2str(clust),' ERSPs for ',STUDY.condition{cond}],'title');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% LOAD raw data:------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

subjs = {'S01','S02','S03','S04','S05','S06','S07','S08','S09','S10','S11','S12','S13'};

subj = 3; % choose subject of interest
cond = 1; % choose condition of interest

load_string = [basedir,subjs{subj},'\',STUDY.datasetinfo(1).filename(1:end-4),'.icaersp'];
%load_string = [basedir,subjs{subj},'\',STUDY.datasetinfo(1).filename(1:end-4),'.icaitc'];
%load_string = [basedir,subjs{subj},'\',STUDY.datasetinfo(1).filename(1:end-4),'.icaspec'];
%load_string = [basedir,subjs{subj},'\',STUDY.datasetinfo(1).filename(1:end-4),'.icatopo'];

RAWdata = load('-mat',load_string);

% Mask an ERSP using calculated bootstrap limits
ic = 8; % choose IC to plot (must have been calculated for this subj)
oneic = ['RAWdata.comp',int2str(ic),'_ersp'];
oneic = eval(oneic);
onebase = ['RAWdata.comp',int2str(ic),'_erspbase']; % to see the baseline spectrum
onebase = eval(onebase);
oneboot = ['RAWdata.comp',int2str(ic),'_erspboot'];
oneboot = eval(oneboot);

maskERSP = oneic;
maskERSP(find(oneic > repmat(oneboot(:,1),[1 size(oneic,2)])& oneic < repmat(oneboot(:,2),[1 size(oneic,2)]))) = 0;

clim = 4; % set +/- color limits
figure; imagesc(RAWdata.times,RAWdata.freqs,maskERSP,[-clim clim]);
set(gca,'ydir','norm'); 
title(['Subj ',int2str(subj),'; IC ',int2str(ic),'; ',STUDY.condition{cond}]);
cbar;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot ERSP data from RAW files :---------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clust = 4; % cluster to analyze

clear condersp
for cond = 1:length(STUDY.condition)
   clear logersp
    for ic = 1:size(STUDY.cluster(clust).sets,2)    
       setidx = STUDY.cluster(clust).sets(cond,ic);
       comp = STUDY.cluster(clust).comps(ic);
       load_string = [STUDY.datasetinfo(setidx).filepath,'\',STUDY.datasetinfo(setidx).filename(1:end-4),'.icaersp'];
       RAWdata = load('-mat',load_string);
       oneic = ['RAWdata.comp',int2str(comp),'_ersp'];
       oneic = eval(oneic);
       logersp(:,:,ic) = oneic;
       condersp{cond} = logersp;
       subjids{ic} = STUDY.datasetinfo(setidx).subject;
       ics(1,ic) = comp;
    end;
end;
% Plot the results:
figure; pl = 1; clim = 4; 
row = 5; col = length(STUDY.condition);
for ic = 1:size(STUDY.cluster(clust).sets,2)  
   if pl > row*col
      figure; pl=1;
   end;
   for cond = 1:length(STUDY.condition)
      sbplot(row,col,pl);pl=pl+1;
   imagesc(RAWdata.times,RAWdata.freqs,condersp{cond}(:,:,ic),[-clim clim]);
   set(gca,'ydir','norm'); title([subjids{ic},'; IC ',int2str(ics(ic)),'; ',STUDY.condition{cond}]);
   end;
end;cbar;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ERSP Time/Frequency window analysis :---------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clust = 5; % cluster to analyze
tlims = [500 1000]; % time limits (ms)
flims = [9 12];     % frequency limits (Hz)

clear leg logersp tfdat
for cond = 1:length(STUDY.condition)
    for ic = 1:size(STUDY.cluster(clust).sets,2)    
       setidx = STUDY.cluster(clust).sets(cond,ic);
       comp = STUDY.cluster(clust).comps(ic);
       load_string = [STUDY.datasetinfo(setidx).filepath,'\',STUDY.datasetinfo(setidx).filename(1:end-4),'.icaersp'];
       RAWdata = load('-mat',load_string);
       oneic = ['RAWdata.comp',int2str(comp),'_ersp'];
       oneic = eval(oneic);
       logersp(:,:,ic) = oneic;
       tms = find(RAWdata.times>tlims(1)&RAWdata.times<tlims(2));
       frs = find(RAWdata.freqs>flims(1)&RAWdata.freqs<flims(2));
       tfdat(cond,ic) = mean(mean(logersp(frs,tms,ic)));
       leg{ic} = ['IC ',num2str(STUDY.cluster(clust).comps(ic))];
    end;
end;

figure; 
ph = bar([1:length(STUDY.condition)],tfdat);
set(gca,'xticklabel',STUDY.condition);
legend(leg)
title([int2str(flims(1)),' to ',int2str(flims(2)),' Hz power between ',int2str(tlims(1)),' and ',int2str(tlims(2)),' ms post-stimulus']);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SCALP ERPs and cluster contributions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STUDY envtopo 
%(THIS FUNCTION IS STILL UNDER CONSTRUCTION, PLEASE CHECK IN
% TO EEGLAB DEVELOPMENTS TO SEE WHEN IT WILL BE AVAILABLE)

% first plot topo maps if you haven't already:
STUDY = std_topoplot(STUDY,ALLEEG,'clusters',[3:length(STUDY.cluster)]); % calculate 'topopol' field 
% Cluster envtopo VARIABLES:------------
tmlims = [70 200]; % [minms maxms] Time interval within which to find largest clusters
plotconds = [1,2,3]; % list condition indices to plot
plotlims = [-50 400]; % [minms maxms] Full time interval to plot
artifact_cls = [13]; % list artifact clusters to remove for plotting
bsln = [-100 0];
%plot_clusters = [7,9,14,21]; % list clusters of interest to plot, or (neg) number to plot
plot_clusters = -4; % For example, plot the 3 largest contributors
%----------------------------------------------------------------
% WARNING: this function takes a LONG time to calculate, select fewer
% conditions or just be patient!!
% Plot specified (or largest) clusters in a time period:------------------------
std_envtopo(STUDY, ALLEEG,'clustnums',plot_clusters,'conditions',plotconds,'env_erp','all','only_clust','on','timerange',plotlims,'subclus',artifact_cls,'limcontrib', tmlims,'baseline',[-100 0]);

% Plot DIFFERENCE between 2 conditions:--------------------------------
tmlims = [200 500]; % [minms maxms] Time interval within which to find largest clusters
plotlims = [-50 600]; % [minms maxms] Full time interval to plot
conddiffs = [2 1]; % list 2 conditions to find difference for. (input1 - input2) 
% 'conditions' is automatically set to the clusters in condiffs

std_envtopo(STUDY, ALLEEG,'clustnums',plot_clusters,'env_erp','contrib','only_clust','on','timerange',plotlims,'subclus',artifact_cls,'limcontrib',tmlims,'baseline',bsln,'diff',conddiffs);


% Plot selected clusters instead:--------------------------------
plot_clusters = [2,3,6,12]; 
std_envtopo(STUDY, ALLEEG,'clustnums',plot_clusters,'conditions',plotconds,'env_erp','all','only_clust','on','timerange',plotlims,'subclus',artifact_cls,'baseline',[-100 0],'limcontrib', tmlims);


 