% this page corresponds to Tutorial 9:
% "STUDY analysis II", by Julie Onton
% Below are novice, intermediate and advanced exercises. 
% Try writing the assigned scripts yourself, only using the scripts below
% to help if you get stuck. 
% If you do need to use the scripts below, proceed line-by-line in your 
% chosen section and be sure you understand what each line does before
% pasting it into the Matlab window. 
% * IMPORTANT: directory paths will be specific for your personal computer, 
%              please edit appropriately in all scripts provided in the 
%              variable called 'datpath' ***

% BEGIN STUDY scripting:--------------------------------------------------------------

%%%  Variables:  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

basedir = 'C:\\Documents and Settings\\eeglab\\My Documents\\MATLAB\\EEGLAB_Workshop\\STUDY\\'; % make specific to your data path
setnames = {'Memorize.set','Ignore.set','Probe.set'};
subjs = {'S01','S02','S03','S04','S05','S06','S07','S08','S09','S10','S11','S12','S13'};
studyname = 'Sternberg';
taskname = 'Sternberg';
savename = 'stern.study';

%%% Open eeglab:
[ALLEEG EEG CURRENTSET ALLCOM] = eeglab; % open eeglab
% make sure memory options are for only one dataset up at once:
pop_editoptions( 'option_storedisk', 1, 'option_savetwofiles', 1, 'option_saveica', 1, 'option_single', 0, 'option_memmapdata', 0, 'option_computeica', 1, 'option_scaleicarms', 0, 'option_rememberfolder', 1);% saves a file 'eeg_options.m' to your current working directory

%load your STUDY:
[STUDY ALLEEG] = pop_loadstudy('filename', savename, 'filepath', basedir);
CURRENTSTUDY = 1; EEG = ALLEEG; CURRENTSET = [1:length(EEG)];
eeglab redraw
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PLOT Cluster ERP image 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plot all mean maps to get topo polarity:
STUDY = std_topoplot(STUDY,ALLEEG,'clusters',[2:length(STUDY.cluster)]);close % calculate 'topopol'
% requires memory options set to pre-calculate ica activations
clust = 7;  % choose a cluster
cond = 3; % choose a condition (from STUDY.condition)

% ERP image paramters:
smoothby  = 40; 
PWRcolrlim = 6; 
ACTcolrlim = 2.5; 
tmlims = [-100 800];
% choose to sort by:-------------------------------------------
sortby = 'nosort'; % no sort
%  <OR>
sortby = 'rt';  % reaction times

% collect activations (correctly oriented) for all cluster ICs:
condttl = STUDY.condition{cond};
CURRENTSTUDY = 1; EEG = ALLEEG; CURRENTSET = [1:length(EEG)];    
rts = []; clear winvs acts chans
for ic = 1:length(STUDY.cluster(clust).comps)
    setidx = STUDY.cluster(clust).sets(cond,ic);
    comp = STUDY.cluster(clust).comps(ic);
    [ALLEEG EEG CURRENTSET] = pop_newset(ALLEEG, EEG, CURRENTSET, 'retrieve',setidx, 'study',CURRENTSTUDY);        
    for ep = 1:length(EEG.epoch)
        pos = find(ismember(EEG.epoch(ep).eventtype,'rt'));
        if ~isempty(pos)
            rts = [rts EEG.epoch(ep).eventlatency{pos}];
        else
            rts = [rts 0];% if no rt, make rt 0 ms
        end;
    end;   
    rmsuv = sqrt(mean(ALLEEG(setidx).icawinv(:,comp).^2));% RMS at scalp
    % collect winv, oriented by 'topopol' field in STUDY:
    winvs{ic} = ALLEEG(setidx).icawinv(:,comp)*STUDY.cluster(clust).topopol(ic);
    % collect activations, oriented by 'topopol' SAME field in STUDY:
    acts{ic} = ALLEEG(setidx).icaact(comp,:,:)*rmsuv*STUDY.cluster(clust).topopol(ic);
    chans{ic} = ALLEEG(setidx).chanlocs; 
end;    
allacts = zeros(1,length(ALLEEG(1).times),0);
for ic = 1:length(STUDY.cluster(clust).comps)
    allacts(:,:,end+1:end+size(acts{ic},3)) = acts{ic};
end;    
allacts = squeeze(allacts); % makes a frames x trials matrix
if strcmp(sortby,'rt')
    sortvar = rts;  % reaction times
else
    sortvar = ones(1,size(allacts,2)); % no sort
end

% PLOT activations:--------------
figure;[outdata,outvar,outtrials,limits,axhndls, erp,amps,cohers,cohsig,ampsig,outamps,phsangls,phsamp,sortidx,erpsig] =erpimage( allacts, sortvar, linspace(EEG.xmin*1000, EEG.xmax*1000, EEG.pnts), ['Cluster ',int2str(clust),'-',condttl], smoothby, 1 ,'yerplabel','','erp','limits',[tmlims NaN NaN NaN NaN NaN NaN] ,'cbar','caxis',[-ACTcolrlim ACTcolrlim],'coher', [9 12 .01]);

% PLOT power:-------------------
figure;[outdata,outvar,outtrials,limits,axhndls, erp,amps,cohers,cohsig,ampsig,outamps,phsangls,phsamp,sortidx,erpsig] = erpimage( allacts, sortvar, linspace(EEG.xmin*1000, EEG.xmax*1000, EEG.pnts), ['Cluster ',int2str(clust),'-',condttl], smoothby, 1 ,'yerplabel','','erp','limits',[tmlims NaN NaN NaN NaN NaN NaN] ,'cbar','caxis',[-PWRcolrlim PWRcolrlim],'coher', [9 12 .01],'plotamps');

% PHASE Sort activations:--------------
ms_center = 250; % window center at 250 ms
topphase = 180; % phase at the top of image 
figure;[outdata,outvar,outtrials,limits,axhndls, erp,amps,cohers,cohsig,ampsig,outamps,phsangls,phsamp,sortidx,erpsig] =erpimage( allacts, sortvar, linspace(EEG.xmin*1000, EEG.xmax*1000, EEG.pnts), ['Cluster ',int2str(clust),'-',condttl], smoothby, 1 ,'yerplabel','','erp','limits',[tmlims NaN NaN NaN NaN NaN NaN] ,'cbar','caxis',[-ACTcolrlim ACTcolrlim],'phasesort' , [ms_center 0 9 12 topphase],'coher', [9 12 .01]);

% PHASE Sort, plot power:--------------
ms_center = 250; % window center at 250 ms
topphase = 180; % phase at the top of image 
figure;[outdata,outvar,outtrials,limits,axhndls, erp,amps,cohers,cohsig,ampsig,outamps,phsangls,phsamp,sortidx,erpsig] =erpimage( allacts, sortvar, linspace(EEG.xmin*1000, EEG.xmax*1000, EEG.pnts), ['Cluster ',int2str(clust),'-',condttl], smoothby, 1 ,'yerplabel','','erp','limits',[tmlims NaN NaN NaN NaN NaN NaN] ,'cbar','caxis',[-ACTcolrlim ACTcolrlim],'phasesort' , [ms_center 0 9 12 topphase],'coher', [9 12 .01],'plotamps');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CROSS coherence between clusters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clust1 = 10; % choose 2 clusters to test coherence between
clust2 = 17;
% Crossf parameters:--------------------------------
type = 'phasecoher';
alpha = .01;
cycles = [3 .5]; % wavelet cycles 
freqscale = 'log';
frqlim = [3 30];% calculation frequency limits in Hz
tmlims = [-100 1000]; % [min max] times in ms for window

% WARNING: this function takes a LONG time to calculate, select fewer
% conditions or just be patient!!
for cond = 1:size(STUDY.cluster(clust1).sets,1)
    ttls = cell(1,0);allrts = cell(1,0); p=1;
    for ic = 1:length(STUDY.cluster(clust1).comps)
        setidx = STUDY.cluster(clust1).sets(cond,ic);
        comp1 = STUDY.cluster(clust1).comps(ic);
        [ALLEEG EEG CURRENTSET] = pop_newset(ALLEEG, EEG, CURRENTSET, 'retrieve',setidx, 'study',CURRENTSTUDY);  
        subjmatch = find(STUDY.cluster(clust2).sets(cond,:) == setidx);
        for c = 1:length(subjmatch)                
            comp2 = STUDY.cluster(clust2).comps(subjmatch(c));
            [coh(:,:,p),mcoh,times,freqs,cohboot(:,p),cohang(:,:,p)] = newcrossf(EEG.icaact(comp1,:) ,EEG.icaact(comp2,:), EEG.pnts, [EEG.xmin*1000 EEG.xmax*1000], EEG.srate, cycles, 'alpha', alpha,'winsize',EEG.srate,'newfig','off','type',type,'freqs',frqlim,'freqscale' ,freqscale,'savecoher',0 , 'plotamp' ,'off','plotphase' ,'off' );  p = p+1;
            ttls{end+1} = [STUDY.datasetinfo(setidx).subject,': ICs ',int2str(comp1),'-',int2str(comp2)];
        end            
    end;
    condcohs{cond} = coh;
    condboots{cond} = cohboot;
    condang{cond} = cohang;
end;
% PLOT the cross coherence results----------------------------
row = 2; col = 2; clim = [0 .2]; % set color limit here
tms = find(times>tmlims(1)&times<tmlims(2));
frs = find(freqs>frqlim(1)&freqs<frqlim(2));
figure; pl = 1;
for cond = 1:length(condcohs)
    sbplot(row,col,pl); 
    clear cohs
    for p = 1:size(coh,3)
        tmpcoh = condcohs{cond}(:,:,p);
        %tmpcoh(find(tmpcoh<repmat(condboots{cond}(:,p),[1 size(tmpcoh,2)]))) = 0;%comment to unmask
        cohs(:,:,p) = tmpcoh;
    end;                
    maskedmat = mean(cohs,3); 
    imagesclogy(times(tms),freqs(frs),maskedmat(frs,tms),clim); 
    set(gca,'ydir','norm');hold on;
    plot([0 0],[get(gca,'ylim')],'k-');  
    ylabel('Frequency (Hz)'); 
    if pl > 2
        xlabel('Latency (ms)');
    end;
    title([STUDY.condition{cond}]);   
    if cond == length(STUDY.condition)
        cbar; % include a colorbar for plotted coh values
    end;pl = pl+1; 
end;
textsc(['Cluster cross coherence between Clusts ',int2str(clust1),' and ',int2str(clust2)],'title');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DENSITY plots
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cond=1;% conds 1 and 2 have dipole info

clust = 4;

dipsources = struct('posxyz',[],'momxyz',[],'rv',[]);nd1 = 1;
nowidx=0; % initialize
for ic = 1:length(STUDY.cluster(clust).comps)
  setidx = STUDY.cluster(clust).sets(cond,ic);
  comp = STUDY.cluster(clust).comps(ic);
  if setidx ~= nowidx % don't call in if already active
    [ALLEEG EEG CURRENTSET] = pop_newset(ALLEEG, EEG, CURRENTSET, 'retrieve',setidx, 'study',CURRENTSTUDY);  nowidx = setidx;
  end;
  model = EEG.dipfit.coordformat;
  dipsources(1,nd1).posxyz = EEG.dipfit.model(comp).posxyz;
  dipsources(1,nd1).momxyz = EEG.dipfit.model(comp).momxyz;
  dipsources(1,nd1).rv = EEG.dipfit.model(comp).rv;  nd1=nd1+1;
end;
dipoledensity(dipsources , 'method','alldistance','methodparam',10,...
   'coordformat',model); 
% compare with:
dipplot(dipsources,'normlen','on','coordformat',model);

% Or calculate for output:
[dens3d, mri]=dipoledensity(dipsources , 'method','alldistance',...
   'methodparam',10,'coordformat',model);
% replot (here you have more plotting parameters possible):
% >> help mri3dplot
figure; mri3dplot(dens3d,mri);
